#!/usr/bin/env python
# -*- coding: cp1252 -*-
##
##    Copyright (C) 2005 manatlan manatlan[at]gmail(dot)com
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 2 only.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

"""Plugin to upload some photos to a flickr account"""
from __main__ import JPlugin,_

from winflickr import Winflickr

# imports for uploader
import urllib2, mimetools, mimetypes, sys
from xml.dom.minidom import parseString

__name__ = "uploadToFlickr"
__author__ = "manatlan"
__version__ = "1.0"
__alter__ = False          # to prevent jbrout if this plugin can alter DB or photos
menuEntries= [(8512,_("Upload to flickr"),"go")]



class Create(JPlugin):

    """
    self.conf                           : indexer on self configuration
    self.showProgress(cur,max,msg)      : show progress bar
    self.parent                         : parent gtk window
    """

    def init(self):
        pass

    def select(self,cde,list):
        # default values
        email = self.conf["email"] or ""
        password = self.conf["password"] or ""
        privacy = int(self.conf["privacy"] or "0")

        # pass params to the window
        Winflickr.params=(email,password,privacy)

        win = Winflickr()
        win.run()

        # analyze response
        if win.response:
            email,password,privacy = win.response

            # store default values
            self.conf["email"]=email
            self.conf["password"]=password
            self.conf["privacy"]=privacy

            # and batch the upload
            try:
                for i in list:
                    info = "%d/%d" % (list.index(i)+1, len(list))
                    self.showProgress( list.index(i), len(list) , _("uploading")+" "+info )
                    err=upload2flickr(email,password,i.file,i.comment,i.tags,privacy)
                    if err:
                        self.MessageBox("Uploader error",err)
                        break
            finally:
                self.showProgress()

            return True
        else:
            return False


#===============================================================================
#===============================================================================
#===============================================================================


# This code is from www.voidspace.org.uk/atlantibots/pythonutils.html
def encode_multipart_formdata(fields, files, BOUNDARY = '-----'+mimetools.choose_boundary()+'-----'):
    """ Encodes fields and files for uploading.
    fields is a sequence of (name, value) elements for regular form fields - or a dictionary.
    files is a sequence of (name, filename, value) elements for data to be uploaded as files.
    Return (content_type, body) ready for urllib2.Request instance
    You can optionally pass in a boundary string to use or we'll let mimetools provide one.
    """
    CRLF = '\r\n'
    L = []
    if isinstance(fields, dict):
        fields = fields.items()
    for (key, value) in fields:
        L.append('--' + BOUNDARY)
        L.append('Content-Disposition: form-data; name="%s"' % key)
        L.append('')
        L.append(value)
    for (key, filename, value) in files:
        filetype = mimetypes.guess_type(filename)[0] or 'application/octet-stream'
        L.append('--' + BOUNDARY)
        L.append('Content-Disposition: form-data; name="%s"; filename="%s"' % (key, filename))
        L.append('Content-Type: %s' % filetype)
        L.append('')
        L.append(value)
    L.append('--' + BOUNDARY + '--')
    L.append('')
    body = CRLF.join(L)
    content_type = 'multipart/form-data; boundary=%s' % BOUNDARY        # XXX what if no files are encoded
    return content_type, body

def build_request(theurl, fields, files, txheaders=None):
    """Given the fields to set and the files to encode it returns a fully formed urllib2.Request object.
    You can optionally pass in additional headers to encode into the opject. (Content-type and Content-length will be overridden if they are set).
    fields is a sequence of (name, value) elements for regular form fields - or a dictionary.
    files is a sequence of (name, filename, value) elements for data to be uploaded as files.
    """
    content_type, body = encode_multipart_formdata(fields, files)
    if not txheaders: txheaders = {}
    txheaders['Content-type'] = content_type
    txheaders['Content-length'] = str(len(body))
    return urllib2.Request(theurl, body, txheaders)


def upload2flickr(email,password,filepath,comment,tags,privacy):
    """ upload the photo "filepath" with comment "comment", and tags "tags"
        to flickr ! thx to Michele Campeotto(micampe@micampe.it) for inspiration
        see http://micampe.it/things/flickruploadr

        privacy in [0,1,2] (public, friend, family)
    """
    assert type(filepath)==unicode
    filepath = filepath.encode("utf_8")
    UPLOADER_URL = "http://www.flickr.com/tools/uploader_go.gne"
    filename = filepath.split("/")[-1]

    INFO = {"email": email,
            "password": password,
            "title": filename,
            "description": comment,
            "tags": ",".join(tags),
            "is_public": (privacy==0 and "1" or "0"),
            "is_friend": (privacy==1 and "1" or "0"),
            "is_family": (privacy==2 and "1" or "0")
           }

    photo = ('photo', filename, open(filepath).read())

    try:
        xml = urllib2.urlopen(build_request(UPLOADER_URL, INFO, (photo,))).read()
        dom = parseString(xml)
        if dom.getElementsByTagName("status")[0].childNodes[0].data=="ok":
            return ""
        else:
            return dom.getElementsByTagName("verbose")[0].childNodes[0].data
    except:
        return "flickr communication trouble : "+str(sys.exc_info()[0])

